static const char *RcsId = "$Header: /home/cvsadm/cvsroot/fermi/servers/adio/src/Adio.cpp,v 1.2 2013-10-29 11:27:03 graziano Exp $";
//+=============================================================================
//
// file :         Adio.cpp
//
// description :  C++ source for the Adio and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                Adio are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: graziano $ Giulio Gaio
//
// $Revision: 1.2 $ 1.0
//
// $Log: Adio.cpp,v $
// Revision 1.2  2013-10-29 11:27:03  graziano
// Tango 8, Device_4Impl
//
// Revision 1.1  2008/06/24 10:24:38  graziano
// first commit
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondance
//	between commands and method's name.
//
//  Command's name      |  Method's name
//	----------------------------------------
//  State               |  dev_state()
//  Status              |  dev_status()
//  EnabIrqMask         |  enab_irq_mask()
//  DisabIrqMask        |  disab_irq_mask()
//  SetDebounce         |  set_debounce()
//  GetDebounce         |  get_debounce()
//  ConfigIOMask        |  config_iomask()
//  GetDigInMask        |  get_dig_in_mask()
//  DigIn               |  dig_in()
//  SetDigOutMask       |  set_dig_out_mask()
//  GetDigOutMask       |  get_dig_out_mask()
//  DigOut              |  dig_out()
//  DigOn               |  dig_on()
//  DigOff              |  dig_off()
//  DigPulse            |  dig_pulse()
//  GetDacCtrl          |  get_dac_ctrl()
//  SetDcDac            |  set_dc_dac()
//  GetDcDac            |  get_dc_dac()
//  SetAcDac            |  set_ac_dac()
//  GetAcDac            |  get_ac_dac()
//  GetAdc              |  get_adc()
//  GetAdcVolt          |  get_adc_volt()
//  SetDacLowTh         |  set_dac_low_th()
//  GetDacLowTh         |  get_dac_low_th()
//  SetDacHighTh        |  set_dac_high_th()
//  GetDacHighTh        |  get_dac_high_th()
//  GetSetPnt           |  get_set_pnt()
//  GetPinConnector     |  get_pin_connector()
//  GetAcDacVolt        |  get_ac_dac_volt()
//  GetConfigIOMask     |  get_config_iomask()
//  SetAcDacVolt        |  set_ac_dac_volt()
//  SetDcDacVolt        |  set_dc_dac_volt()
//  RegisterChannels    |  register_channels()
//  UnregisterChannels  |  unregister_channels()
//  GetChannelsConfig   |  get_channels_config()
//
//===================================================================


#include <tango.h>
#include <Adio.h>
#include <AdioClass.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <adio.h>

#define OUTPUT		0
#define INPUT		1
#define NOTUSED		2

//#define _REGISTER_CHANNELS      1

namespace Adio_ns
{

//+----------------------------------------------------------------------------
//
// method : 		Adio::Adio(string &s)
// 
// description : 	constructor for simulated Adio
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
Adio::Adio(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

Adio::Adio(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

Adio::Adio(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		Adio::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void Adio::delete_device()
{
	//	Delete device's allocated object
	if (DigOut) {
		delete DigOut;
		DigOut = 0;
	}
	if (DigIn) {
		delete DigIn;
		DigIn = 0;
	}	
	
	if (fd > 0)
		close(fd);

}

//+----------------------------------------------------------------------------
//
// method : 		Adio::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void Adio::init_device()
{
	INFO_STREAM << "Adio::Adio() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();
	
	TangoSys_MemStream out_stream;
	int err;
	long vmeaddr; 
	unsigned short val;
	
	if ((fd = open(adioname.c_str(),O_RDWR)) < 0) {
		out_stream << "Error opening " << adioname << ends;
		cout << out_stream.str() << endl;
		set_state(Tango::DISABLE); 
		set_status(out_stream.str());
	}	
	else { 
		ioctl(fd,ADIO_VMEADDR,&vmeaddr);
		char addr[20];sprintf(addr,"0x%x",vmeaddr);
		out_stream << "Open " << adioname << "; vmeaddr " << addr << ends;	
		cout << out_stream.str() << endl;
		set_state(Tango::ON); 
		set_status(out_stream.str());	
		// set high threshold
		val = 0x7fff;
		ioctl(fd,ADIO_WRHIGHT,&val);
		// set low threshold
		val = 0x8000;
		ioctl(fd,ADIO_WRLOWT,&val);					
	}
	stringstream hexstr;
	hexstr << iomask;
	if(iomask.substr(0,2) == "0x")
		hexstr >> hex >> val;	
	else
		hexstr >> val;
	//val = iomask;
	DEBUG_STREAM << "init_device: setting iomask=0x" << hex << val << endl;
	if (ioctl(fd,ADIO_DIGIT,&val)) 
	{
		Tango::Except::throw_exception (
		(const char *)"Can't configure iomask",
		(const char *)"IOctl error",
		(const char *)"Adio::init_device()");
	}
	
	DigOut = new Tango::DevShort(ADIO_OUT_CHAN);
	if(DigOut == 0)
	{
		Tango::Except::throw_exception (
		(const char *)"OUT OF MEMORY",
		(const char *)"out of memory error",
		(const char *)"Adio::init_device()");
	}

	DigIn = new Tango::DevShort(ADIO_IN_CHAN);
	if(DigIn == 0)
	{
		Tango::Except::throw_exception (
		(const char *)"OUT OF MEMORY",
		(const char *)"out of memory error",
		(const char *)"Adio::init_device()");
	}

	chanConfig.length(ADIO_IN_CHAN);

#ifdef _REGISTER_CHANNELS
	for(int i=0;i<ADIO_IN_CHAN;i++)
		chanConfig[i]=NOTUSED;		//control on registered channel 
#else
	for(int i=0;i<ADIO_IN_CHAN;i++)  //considers all channels as used
	{
		chanConfig[i] = (val >> i) & 0x0001;  //so controls only if channels are right configured as in or out
	}
#endif		

}


//+----------------------------------------------------------------------------
//
// method : 		Adio::readDeviceProperies()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void Adio::get_device_property()
{
	//	Initialize your default values here.
	//------------------------------------------
	ioconnector.resize(14);
	ioconnector[0] = "A2: I/O + || C2: I/O -";
	ioconnector[1] = "A4: I/O + || C4: I/O -";
	ioconnector[2] = "A6: I/O + || C6: I/O -";
	ioconnector[3] = "A8: I/O + || C8: I/O -";
	ioconnector[4] = "A10: I/O + || C10: I/O -";
	ioconnector[5] = "A12: I/O + || C12: I/O -";
	ioconnector[6] = "A14: I/O + || C14: I/O -";
	ioconnector[7] = "A16: I/O + || C16: I/O -";
	ioconnector[8] = "A18: I + || C18: I -";
	ioconnector[9] = "A20: I + || C20: I -";
	ioconnector[10] = "A22: I + || C22: I -";
	ioconnector[11] = "A24: I + || C24: I -";									
	ioconnector[12] = "A26: DAC + || C26: DAC -";
	ioconnector[13] = "A30: ADC + || C30: ADC -";	
	iomask = 0xffff;

	//	Read device properties from database.(Automatic code generation)
	//-------------------------------------------------------------
	if (Tango::Util::instance()->_UseDb==false)
		return;
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("Adioname"));
	dev_prop.push_back(Tango::DbDatum("Ioconnector"));
	dev_prop.push_back(Tango::DbDatum("Iomask"));

	//	Call database and extract values
	//--------------------------------------------
	get_db_device()->get_property(dev_prop);
	AdioClass	*ds_class =
		(static_cast<AdioClass *>(get_device_class()));
	int	i = -1;
	//	Extract Adioname value
	if (dev_prop[++i].is_empty()==false)	dev_prop[i]  >>  adioname;
	else
	{
		//	Try to get value from class property
		Tango::DbDatum	cl_prop = ds_class->get_class_property(dev_prop[i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  adioname;
	}

	//	Extract Ioconnector value
	if (dev_prop[++i].is_empty()==false)	dev_prop[i]  >>  ioconnector;
	else
	{
		//	Try to get value from class property
		Tango::DbDatum	cl_prop = ds_class->get_class_property(dev_prop[i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  ioconnector;
	}

	//	Extract Iomask value
	if (dev_prop[++i].is_empty()==false)	dev_prop[i]  >>  iomask;
	else
	{
		//	Try to get value from class property
		Tango::DbDatum	cl_prop = ds_class->get_class_property(dev_prop[i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  iomask;
	}

	//	End of Automatic code generation
	//-------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		Adio::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void Adio::always_executed_hook()
{
	
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::enab_irq_mask
 *
 *	description:	method to execute "EnabIrqMask"
 *	Set enabling irq configuration mask.
 *	bit 0-7: 0->1 interrupt generation
 *	bit 8-15: 1->0 interrupt generation
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Adio::enab_irq_mask(Tango::DevUShort argin)
{
	DEBUG_STREAM << "Adio::enab_irq_mask(): entering... !" << endl;

	//	Add your own code to control device here
	short val = argin;
	int err;	
	
	// system call
	if ((err = ioctl(fd,ADIO_ENABINT,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"EnabIrqMaskAdioException",
			out_stream.str(),
			(const char *) "Adio::enab_irq_mask()", Tango::ERR);
	}

	set_state(Tango::ON);

}

//+------------------------------------------------------------------
/**
 *	method:	Adio::disab_irq_mask
 *
 *	description:	method to execute "DisabIrqMask"
 *	Set disabling irq configuration mask.
 *	bit 0-7: 0->1 interrupt generation
 *	bit 8-15: 1->0 interrupt generation
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Adio::disab_irq_mask(Tango::DevUShort argin)
{
	DEBUG_STREAM << "Adio::disab_irq_mask(): entering... !" << endl;

	//	Add your own code to control device here
	short val = argin;
	int err;	
	
	// system call
	if ((err = ioctl(fd,ADIO_DISABINT,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"DisabIrqMaskAdioException",
			out_stream.str(),
			(const char *) "Adio::disab_irq_mask()", Tango::ERR);
	}
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::set_debounce
 *
 *	description:	method to execute "SetDebounce"
 *	Set debounce factor.
 *	Allowed values are 0,1,2,3.
 *
 * @param	argin	debounce factor
 *
 */
//+------------------------------------------------------------------
void Adio::set_debounce(Tango::DevShort argin)
{
	DEBUG_STREAM << "Adio::set_debounce(): entering... !" << endl;

	//	Add your own code to control device here
	short val = argin;
	int err;	

	switch(val) {
		case 0:
		case 1:
		case 2:
		case 3:break;
		default:
			Tango::Except::throw_exception(
				(const char *)"SetDebounceAdioException",
				(const char *)"Debounce factor unknown (0,1,2,3)",
				(const char *) "Adio::set_debounce()", Tango::ERR);
	}
	
	// system call
	if ((err = ioctl(fd,ADIO_WRDEBNCE,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"SetDebounceAdioException",
			out_stream.str(),
			(const char *) "Adio::set_debounce()", Tango::ERR);
	}

	set_state(Tango::ON);
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_debounce
 *
 *	description:	method to execute "GetDebounce"
 *	Get debounce factor.
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevShort Adio::get_debounce()
{
	Tango::DevShort	argout ;
	DEBUG_STREAM << "Adio::get_debounce(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_RDDEBNCE,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"GetDebounceAdioException",
			out_stream.str(),
			(const char *) "Adio::get_debounce()", Tango::ERR);
	}
	
	argout = val;
	set_state(Tango::ON);
	
	return argout;
}
//+------------------------------------------------------------------
/**
 *	method:	Adio::config_iomask
 *
 *	description:	method to execute "ConfigIOMask"
 *	Config input output channels: 0=output 1=input
 *
 * @param	argin	iomask
 *
 */
//+------------------------------------------------------------------
void Adio::config_iomask(Tango::DevUShort argin)
{
	DEBUG_STREAM << "Adio::config_iomask(): entering... !" << endl;

	//	Add your own code to control device here
	unsigned short val = argin;
	int err;	
	//enable all channel	
	for(int i=0;i<ADIO_IN_CHAN;i++)
	{
		if(chanConfig[i] != NOTUSED)
			chanConfig[i] = (val >> i) & 0x0001;
	}
	stringstream hexstr;
	hexstr << "0x" << hex << val;
	hexstr >> iomask;
	DEBUG_STREAM << "Adio::config_iomask: IOMask updated to: " << iomask << endl;
	// system call
	if ((err = ioctl(fd,ADIO_DIGIT,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"ConfigIOMaskAdioException",
			out_stream.str(),
			(const char *) "Adio::config_iomask()", Tango::ERR);
	}

	set_state(Tango::ON);
		
}


//+------------------------------------------------------------------
/**
 *	method:	Adio::dig_in
 *
 *	description:	method to execute "DigIn"
 *	Get a digital input level.
 *
 * @param	argin	array of channels
 * @return	array of levels
 *
 */
//+------------------------------------------------------------------
Tango::DevVarShortArray *Adio::dig_in(const Tango::DevVarShortArray *argin)
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarShortArray	*argout  = new Tango::DevVarShortArray();

	DEBUG_STREAM << "Adio::dig_in(): entering... !" << endl;

	//	Add your own code to control device here
	int num_chan = argin->length();
	short val;
	int err;
	
	if (num_chan > ADIO_IN_CHAN) {
		TangoSys_MemStream out_stream;
		out_stream << "Too many requested channels" << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(		
			(const char *)"DigInAdioException",
			out_stream.str(),
			(const char *) "Adio::dig_in()", Tango::ERR);
	}
	for (int i = 0; i < num_chan; i++) 
	{
		if((chanConfig[(*argin)[i]] == NOTUSED) || (chanConfig[(*argin)[i]] == OUTPUT))
		{
			TangoSys_MemStream out_stream;
			if(chanConfig[(*argin)[i]] == NOTUSED)
				out_stream << "Channel " << (*argin)[i] << " is not registered to be used" << ends;
			else
				out_stream << "Channel " << (*argin)[i] << " is not configured as input" << ends;
			Tango::Except::throw_exception (
      			(const char *)"Dig In Failure",
      			out_stream.str(),
     			(const char *)"Adio::dig_in", Tango::ERR);
			return argout;	
		}	
	}	
	// system call
	if ((err = ioctl(fd,ADIO_RDDIGIN,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"DigInAdioException",
			out_stream.str(),
			(const char *) "Adio::dig_in()", Tango::ERR);
	}
	
	argout->length(num_chan);
	for (int i = 0; i < num_chan; i++) 
	{
		if (val & (1 << (*argin)[i]))
			DigIn[i] = 1;
		else
			DigIn[i] = 0;		
		(*argout)[i] = DigIn[i];
	}
	
	set_state(Tango::ON);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::set_dig_out_mask
 *
 *	description:	method to execute "SetDigOutMask"
 *	Set digital output mask
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Adio::set_dig_out_mask(Tango::DevUShort argin)
{
	DEBUG_STREAM << "Adio::set_dig_out_mask(): entering... !" << endl;

	//	Add your own code to control device here
	short val = argin;
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_WRDIGO,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"SetDigOutMaskAdioException",
			out_stream.str(),
			(const char *) "Adio::set_dig_out_mask()", Tango::ERR);
	}

	set_state(Tango::ON);
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_dig_out_mask
 *
 *	description:	method to execute "GetDigOutMask"
 *	Get digital output setting mask
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort Adio::get_dig_out_mask()
{
	Tango::DevShort	argout ;
	DEBUG_STREAM << "Adio::get_dig_out_mask(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_RDDIGO,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"GetDigOutMaskAdioException",
			out_stream.str(),
			(const char *) "Adio::get_dig_out_mask()", Tango::ERR);
	}
	
	argout = val;
	set_state(Tango::ON);
	
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::dig_out
 *
 *	description:	method to execute "DigOut"
 *	Get a digital output setting level.
 *
 * @param	argin	channel
 * @return	level
 *
 */
//+------------------------------------------------------------------
Tango::DevVarShortArray *Adio::dig_out(const Tango::DevVarShortArray *argin)
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarShortArray	*argout  = new Tango::DevVarShortArray();

	DEBUG_STREAM << "Adio::dig_out(): entering... !" << endl;

	//	Add your own code to control device here
	int num_chan = argin->length();
	short val;
	int err;
	
	if (num_chan > ADIO_OUT_CHAN) {
		TangoSys_MemStream out_stream;
		out_stream << "Too many requested channels" << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(		
			(const char *)"DigOutAdioException",
			out_stream.str(),
			(const char *) "Adio::dig_out()", Tango::ERR);
	}
		
	// system call
	if ((err = ioctl(fd,ADIO_RDDIGO,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
 		Tango::Except::throw_exception(
			(const char *)"DigOutAdioException",
			out_stream.str(),
			(const char *) "Adio::dig_out()", Tango::ERR);
	}
	
	argout->length(num_chan);
	for (int i = 0; i < num_chan; i++) {
		if (val & (1 << (*argin)[i]))
			DigOut[i] = 1;
		else
			DigOut[i] = 0;		
		(*argout)[i] = DigOut[i];
	}
	
	set_state(Tango::ON);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::dig_on
 *
 *	description:	method to execute "DigOn"
 *	Set channel level to 1.
 *
 * @param	argin	array of channel
 *
 */
//+------------------------------------------------------------------
void Adio::dig_on(const Tango::DevVarShortArray *argin)
{
	DEBUG_STREAM << "Adio::dig_on(): entering... !" << endl;

	//	Add your own code to control device here
	int num_chan = argin->length();
	short val;
	int err;
	
	if (num_chan > ADIO_OUT_CHAN) {
		TangoSys_MemStream out_stream;
		out_stream << "Too many requested channels" << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(		
			(const char *)"DigOnAdioException",
			out_stream.str(),
			(const char *) "Adio::dig_on()", Tango::ERR);
	}

	val = 0;
	for (int i = 0; i < num_chan; i++) 
	{
		if((chanConfig[(*argin)[i]] == NOTUSED) || (chanConfig[(*argin)[i]] == INPUT))
		{
			TangoSys_MemStream out_stream;
			if(chanConfig[(*argin)[i]] == NOTUSED)
				out_stream << "Channel " << (*argin)[i] << " is not registered to be used" << ends;
			else
				out_stream << "Channel " << (*argin)[i] << " is not configured as output" << ends;
			Tango::Except::throw_exception (
      			(const char *)"Dig On Failure",
      			out_stream.str(),
     			(const char *)"Adio::dig_on", Tango::ERR);
			return;	
		}
		val += (1 << (*argin)[i]);
	}	
	// system call
	if ((err = ioctl(fd,ADIO_WRDIGOLON,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
  		Tango::Except::throw_exception(
			(const char *)"DigOnAdioException",
			out_stream.str(),
			(const char *) "Adio::dig_on()", Tango::ERR);
	}

	set_state(Tango::ON);

}

//+------------------------------------------------------------------
/**
 *	method:	Adio::dig_off
 *
 *	description:	method to execute "DigOff"
 *	Set digital output channel to 0.
 *
 * @param	argin	array of channels
 *
 */
//+------------------------------------------------------------------
void Adio::dig_off(const Tango::DevVarShortArray *argin)
{
	DEBUG_STREAM << "Adio::dig_off(): entering... !" << endl;

	//	Add your own code to control device here
	int num_chan = argin->length();
	short val;
	int err;
	
	if (num_chan > ADIO_OUT_CHAN) {
		TangoSys_MemStream out_stream;
		out_stream << "Too many requested channels" << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(		
			(const char *)"DigOffAdioException",
			out_stream.str(),
			(const char *) "Adio::dig_off()", Tango::ERR);
	}

	val = 0;
	for (int i = 0; i < num_chan; i++) 
	{
		if((chanConfig[(*argin)[i]] == NOTUSED) || (chanConfig[(*argin)[i]] == INPUT))
		{
			TangoSys_MemStream out_stream;
			if(chanConfig[(*argin)[i]] == NOTUSED)
				out_stream << "Channel " << (*argin)[i] << " is not registered to be used" << ends;
			else
				out_stream << "Channel " << (*argin)[i] << " is not configured as output" << ends;
			Tango::Except::throw_exception (
      			(const char *)"Dig Off Failure",
      			out_stream.str(),
     			(const char *)"Adio::dig_off", Tango::ERR);
			return;	
		}
		val += (1 << (*argin)[i]);
	}

	// system call
	if ((err = ioctl(fd,ADIO_WRDIGOLOFF,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"DigOffAdioException",
			out_stream.str(),
			(const char *) "Adio::dig_off()", Tango::ERR);
	}

	set_state(Tango::ON);
	
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::dig_pulse
 *
 *	description:	method to execute "DigPulse"
 *	Generate a pulse of 100ms. on a digital channel.
 *
 * @param	argin	array of channels
 *
 */
//+------------------------------------------------------------------
void Adio::dig_pulse(const Tango::DevVarShortArray *argin)
{
	DEBUG_STREAM << "Adio::dig_pulse(): entering... !" << endl;

	//	Add your own code to control device here
	int num_chan = argin->length();
	short val;
	int err;
	
	if (num_chan > ADIO_OUT_CHAN) {
		TangoSys_MemStream out_stream;
		out_stream << "Too many requested channels" << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(		
			(const char *)"DigPulseAdioException",
			out_stream.str(),
			(const char *) "Adio::dig_pulse()", Tango::ERR);
	}

	val = 0;
	for (int i = 0; i < num_chan; i++) 
	{
		if((chanConfig[(*argin)[i]] == NOTUSED) || (chanConfig[(*argin)[i]] == INPUT))
		{
			TangoSys_MemStream out_stream;
			if(chanConfig[(*argin)[i]] == NOTUSED)
				out_stream << "Channel " << (*argin)[i] << " is not registered to be used" << ends;
			else
				out_stream << "Channel " << (*argin)[i] << " is not configured as output" << ends;
			Tango::Except::throw_exception (
      			(const char *)"Dig Pulse Failure",
      			out_stream.str(),
     			(const char *)"Adio::dig_pulse", Tango::ERR);
			return;	
		}	
		val += (1 << (*argin)[i]);
	}

	// system call
	if ((err = ioctl(fd,ADIO_WRDIGOP,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"DigPulseAdioException",
			out_stream.str(),
			(const char *) "Adio::dig_pulse()", Tango::ERR);
	}

	set_state(Tango::ON);
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_dac_ctrl
 *
 *	description:	method to execute "GetDacCtrl"
 *	Get DAC control register.
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevShort Adio::get_dac_ctrl()
{
	Tango::DevShort	argout ;
	DEBUG_STREAM << "Adio::get_dac_ctrl(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_RDDAC,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
  		Tango::Except::throw_exception(
			(const char *)"GetDacCtrlAdioException",
			out_stream.str(),
			(const char *) "Adio::get_dac_ctrl()", Tango::ERR);
	}
	
	argout = val;
	set_state(Tango::ON);
	
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::set_dc_dac
 *
 *	description:	method to execute "SetDcDac"
 *	Set DC DAC component.
 *
 * @param	argin	dc value
 *
 */
//+------------------------------------------------------------------
void Adio::set_dc_dac(Tango::DevShort argin)
{
	DEBUG_STREAM << "Adio::set_dc_dac(): entering... !" << endl;

	//	Add your own code to control device here
	short val = argin;
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_WRDCDAC,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"SetDcDacAdioException",
			out_stream.str(),
			(const char *) "Adio::set_dc_dac()", Tango::ERR);
	}

	set_state(Tango::ON);
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_dc_dac
 *
 *	description:	method to execute "GetDcDac"
 *	Get DC DAC component
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevShort Adio::get_dc_dac()
{
	Tango::DevShort	argout ;
	DEBUG_STREAM << "Adio::get_dc_dac(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;	
	
	if ((err = ioctl(fd,ADIO_RDDCDAC,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
  		Tango::Except::throw_exception(
			(const char *)"GetDcDacAdioException",
			out_stream.str(),
			(const char *) "Adio::get_dc_dac()", Tango::ERR);
	}
	
	argout = val;
	set_state(Tango::ON);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::set_ac_dac
 *
 *	description:	method to execute "SetAcDac"
 *	Set AC DAC component.
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Adio::set_ac_dac(Tango::DevShort argin)
{
	DEBUG_STREAM << "Adio::set_ac_dac(): entering... !" << endl;

	//	Add your own code to control device here
	short val = argin;
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_WRACDAC,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"SetAcDacAdioException",
			out_stream.str(),
			(const char *) "Adio::set_ac_dac()", Tango::ERR);
	}

	set_state(Tango::ON);
	
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_ac_dac
 *
 *	description:	method to execute "GetAcDac"
 *	Get AC DAC component.
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevShort Adio::get_ac_dac()
{
	Tango::DevShort	argout ;
	DEBUG_STREAM << "Adio::get_ac_dac(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;		
	
	if ((err = ioctl(fd,ADIO_RDACDAC,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"GetAcDacAdioException",
			out_stream.str(),
			(const char *) "Adio::get_ac_dac()", Tango::ERR);
	}
	
	argout = val;
	set_state(Tango::ON);
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_adc
 *
 *	description:	method to execute "GetAdc"
 *	Get ADC value.
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevShort Adio::get_adc()
{
	Tango::DevShort	argout;
	DEBUG_STREAM << "Adio::get_adc(): entering... !" << endl;

	//	Add your own code to control device here
	// argin is not used
	short val;
	int err;		
	
	if ((err = ioctl(fd,ADIO_RDADC,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
  		Tango::Except::throw_exception(
			(const char *)"GetAdcAdioException",
			out_stream.str(),
			(const char *) "Adio::get_adc()", Tango::ERR);
	}
	
	argout = val;
	set_state(Tango::ON);
	return argout;

}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_adc_volt
 *
 *	description:	method to execute "GetAdcVolt"
 *	Get ADC value in Volt
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevDouble Adio::get_adc_volt()
{
	Tango::DevDouble	argout ;
	DEBUG_STREAM << "Adio::get_adc_volt(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;		
	
	if ((err = ioctl(fd,ADIO_RDADC,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"GetAdcVoltAdioException",
			out_stream.str(),
			(const char *) "Adio::get_adc_volt()", Tango::ERR);
	}
	
	argout = ((double)val)*10/32768;
	set_state(Tango::ON);
	
	return argout;
	
}



//+------------------------------------------------------------------
/**
 *	method:	Adio::get_set_pnt
 *
 *	description:	method to execute "GetSetPnt"
 *	Get output setpoint.
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevShort Adio::get_set_pnt()
{
	Tango::DevShort	argout ;
	DEBUG_STREAM << "Adio::get_set_pnt(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;		
	
	if ((err = ioctl(fd,ADIO_RDLSETP,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"GetSetPntAdioException",
			out_stream.str(),
			(const char *) "Adio::get_set_pnt()", Tango::ERR);
		set_state(Tango::FAULT);
	}
	
	argout = val;
	set_state(Tango::ON);

	return argout;
}


//+------------------------------------------------------------------
/**
 *	method:	Adio::get_pin_connector
 *
 *	description:	method to execute "GetPinConnector"
 *	This command returns the P2 VME pinout of the ADIO.
 *	Channel 0-7:digital In/Out
 *	Channel 8-11:digital In
 *	Channel 12: DAC
 *	Channel 13: ADC
 *
 * @param	argin	channel
 * @return	pinout
 *
 */
//+------------------------------------------------------------------
Tango::DevString Adio::get_pin_connector(Tango::DevShort argin)
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	DEBUG_STREAM << "Adio::get_pin_connector(): entering... !" << endl;

	//	Add your own code to control device here
	if (argin > 13) {
		TangoSys_MemStream out_stream;
		out_stream << "Invalid number of channel" << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(		
			(const char *)"GetPinConnectorAdioException",
			out_stream.str(),
			(const char *) "Adio::get_pin_connector()", Tango::ERR);
	}
	
	Tango::DevString argout  = new char[ioconnector[argin].size()+1];
	memset(argout,0,ioconnector[argin].size());
	strncpy(argout,ioconnector[argin].c_str(),ioconnector[argin].size());
	argout[ioconnector[argin].size()]=0;

	return argout;

}

//+------------------------------------------------------------------
/**
 *	method:	Adio::set_dac_low_th
 *
 *	description:	method to execute "SetDacLowTh"
 *	Set ADC low threshold.
 *	Default value is 0x8000.
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Adio::set_dac_low_th(Tango::DevShort argin)
{
	DEBUG_STREAM << "Adio::set_dac_low_th(): entering... !" << endl;

	//	Add your own code to control device here
	short val = argin;
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_WRLOWT,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
  		Tango::Except::throw_exception(
			(const char *)"SetDacLowThAdioException",
			out_stream.str(),
			(const char *) "Adio::set_dac_low_th()", Tango::ERR);

	}

	set_state(Tango::ON);

}


//+------------------------------------------------------------------
/**
 *	method:	Adio::get_dac_low_th
 *
 *	description:	method to execute "GetDacLowTh"
 *	Get DAC low threshold.
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevShort Adio::get_dac_low_th()
{
	Tango::DevShort	argout ;
	DEBUG_STREAM << "Adio::get_dac_low_th(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;		
	
	if ((err = ioctl(fd,ADIO_RDLOWT,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
  		Tango::Except::throw_exception(
			(const char *)"GetDacLowThAdioException",
			out_stream.str(),
			(const char *) "Adio::get_dac_low_th()", Tango::ERR);

	}
	
	argout = val;
	set_state(Tango::ON);

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::set_dac_high_th
 *
 *	description:	method to execute "SetDacHighTh"
 *	Set DAC high threshold.
 *	Default value is 0x7fff.
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Adio::set_dac_high_th(Tango::DevShort argin)
{
	DEBUG_STREAM << "Adio::set_dac_high_th(): entering... !" << endl;

	//	Add your own code to control device here
	short val = argin;
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_WRHIGHT,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
  		Tango::Except::throw_exception(
			(const char *)"SetDacHighThAdioException",
			out_stream.str(),
			(const char *) "Adio::set_dac_high_th()", Tango::ERR);

	}

	set_state(Tango::ON);

}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_dac_high_th
 *
 *	description:	method to execute "GetDacHighTh"
 *	Get DAC high threshold.
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevShort Adio::get_dac_high_th()
{
	Tango::DevShort	argout ;
	DEBUG_STREAM << "Adio::get_dac_high_th(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;		
	
	if ((err = ioctl(fd,ADIO_RDHIGHT,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
  		Tango::Except::throw_exception(
			(const char *)"GetDacHighThAdioException",
			out_stream.str(),
			(const char *) "Adio::get_dac_high_th()", Tango::ERR);
	}
	
	argout = val;
	set_state(Tango::ON);
	
	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_ac_dac_volt
 *
 *	description:	method to execute "GetAcDacVolt"
 *	Get DAC AC component in volt
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevDouble Adio::get_ac_dac_volt()
{
	Tango::DevDouble	argout ;
	DEBUG_STREAM << "Adio::get_ac_dac_volt(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;		
	
	if ((err = ioctl(fd,ADIO_RDACDAC,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
  		Tango::Except::throw_exception(
			(const char *)"GetAcDacVoltAdioException",
			out_stream.str(),
			(const char *) "Adio::get_ac_dac_volt()", Tango::ERR);
	}
	
	argout = ((double)val)*10/32768;
	set_state(Tango::ON);
	return argout;
	
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_config_iomask
 *
 *	description:	method to execute "GetConfigIOMask"
 *	Get IO configuration mask.
 *	1=input, 0=output
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort Adio::get_config_iomask()
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "Adio::get_config_iomask(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;		
	
	if ((err = ioctl(fd,ADIO_RDDIGIT,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
  		Tango::Except::throw_exception(
			(const char *)"GetConfigIOMaskAdioException",
			out_stream.str(),
			(const char *) "Adio::get_config_iomask()", Tango::ERR);
	}
	
	argout = val;	
	return argout;

}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_dig_in_mask
 *
 *	description:	method to execute "GetDigInMask"
 *	Get digital input mask
 *
 * @return	mask
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort Adio::get_dig_in_mask()
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "Adio::get_dig_in_mask(): entering... !" << endl;

	//	Add your own code to control device here
	short val;
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_RDDIGIN,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"GetDigInMaskAdioException",
			out_stream.str(),
			(const char *) "Adio::get_dig_in_mask()", Tango::ERR);
	}
	
	argout = val;
	set_state(Tango::ON);

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::set_ac_dac_volt
 *
 *	description:	method to execute "SetAcDacVolt"
 *	Set AC DAC component in Volt
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Adio::set_ac_dac_volt(Tango::DevDouble argin)
{
	DEBUG_STREAM << "Adio::set_ac_dac_volt(): entering... !" << endl;

	//	Add your own code to control device here
	short val = (short)(argin*32768/10);
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_WRACDAC,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"SetAcDacVoltAdioException",
			out_stream.str(),
			(const char *) "Adio::set_ac_dac_volt()", Tango::ERR);

	}
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::set_dc_dac_volt
 *
 *	description:	method to execute "SetDcDacVolt"
 *	Set DC DAC component in Volt.
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Adio::set_dc_dac_volt(Tango::DevDouble argin)
{
	DEBUG_STREAM << "Adio::set_dc_dac_volt(): entering... !" << endl;

	//	Add your own code to control device here
	short val = (short)(argin/32768);
	int err;	

	// system call
	if ((err = ioctl(fd,ADIO_WRACDAC,&val))) { 		
		TangoSys_MemStream out_stream;
		out_stream << adio_get_errmsg(err) << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(
			(const char *)"SetDcDacVoltAdioException",
			out_stream.str(),
			(const char *) "Adio::set_dc_dac_volt()", Tango::ERR);

	}
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::register_channels
 *
 *	description:	method to execute "RegisterChannels"
 *	Set channel listed as used
 *
 * @param	argin	list of channel to set as used
 *
 */
//+------------------------------------------------------------------
void Adio::register_channels(const Tango::DevVarUShortArray *argin)
{
	DEBUG_STREAM << "Adio::register_channels(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	unsigned short val;
	stringstream hexstr;
	hexstr << iomask;
	if(iomask.substr(0,2) == "0x")
		hexstr >> hex >> val;	
	else
		hexstr >> val;

	if (argin->length() > ADIO_IN_CHAN) {
		TangoSys_MemStream out_stream;
		out_stream << "Too many requested channels" << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(		
			(const char *)"TooManyChannel",
			out_stream.str(),
			(const char *) "Adio::register_channels", Tango::ERR);
	}
	
	for(int i=0; i < argin->length(); i++)
	{
		if(chanConfig[(*argin)[i]] == NOTUSED)
			chanConfig[(*argin)[i]] = (val >> (*argin)[i]) & 0x0001;
		else
		{
			TangoSys_MemStream out_stream;
			out_stream << "Channel " << (*argin)[i] << " is already set as used" << ends;			
			Tango::Except::throw_exception (
      			(const char *)"RegisterChannelFailure",
      			out_stream.str(),
     			(const char *)"Adio::register_channels", Tango::ERR);
			return;	
		}
	}
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::unregister_channels
 *
 *	description:	method to execute "UnregisterChannels"
 *	Set channel listed as not used
 *
 * @param	argin	list of channel to set as not used
 *
 */
//+------------------------------------------------------------------
void Adio::unregister_channels(const Tango::DevVarUShortArray *argin)
{
	DEBUG_STREAM << "Adio::unregister_channels(): entering... !" << endl;

	//	Add your own code to control device here
	int err;

	if (argin->length() > ADIO_IN_CHAN) {
		TangoSys_MemStream out_stream;
		out_stream << "Too many requested channels" << ends;
		set_state(Tango::FAULT);
		Tango::Except::throw_exception(		
			(const char *)"TooManyChannel",
			out_stream.str(),
			(const char *) "Adio::unregister_channels", Tango::ERR);
	}

	for(int i=0; i < argin->length(); i++)
	{
		if((chanConfig[(*argin)[i]] != NOTUSED))
		{		
			chanConfig[(*argin)[i]] = NOTUSED;
		}
		else      //if want to set as NOTUSED channel already in this state
		{
			TangoSys_MemStream out_stream;
			out_stream << "Channel " << (*argin)[i] << " is already NOTUSED" << ends;			
			Tango::Except::throw_exception (
      			(const char *)"Unregister Channel Failure",
      			out_stream.str(),
     			(const char *)"Adio::unregister_channels", Tango::ERR);
			return;	
		}
	}
}

//+------------------------------------------------------------------
/**
 *	method:	Adio::get_channels_config
 *
 *	description:	method to execute "GetChannelsConfig"
 *	Return the configuration of each channel: if it is set as input or output, and if it is actually   used or not used.
 *
 * @return	config (in/out, used/not_used) of every digital channel
 *
 */
//+------------------------------------------------------------------
Tango::DevVarStringArray *Adio::get_channels_config()
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarStringArray	*argout  = new Tango::DevVarStringArray();
	DEBUG_STREAM << "Adio::get_channels_config(): entering... !" << endl;

	//	Add your own code to control device here
	string tmp;
	unsigned short val;
	stringstream hexstr;
	hexstr << iomask;
	if(iomask.substr(0,2) == "0x")
		hexstr >> hex >> val;	
	else
		hexstr >> val;
	argout->length(ADIO_IN_CHAN);
	for(int i=0; i < ADIO_IN_CHAN; i++)
	{
		switch(chanConfig[i])
		{
			case INPUT:
				tmp = "IN";
				break;
			case OUTPUT:
				tmp = "OUT";
				break;
			case NOTUSED:
				tmp = "NOT USED";
				if(((val >> i) & 0x0001) == INPUT )
					tmp += " (IN)";
				else
					tmp += " (OUT)";
				break;
		}
		(*argout)[i] = CORBA::string_dup(tmp.c_str());
	}
	return argout;
}

}	//	namespace
